const St = imports.gi.St;
const Main = imports.ui.main;
const Lang = imports.lang;
const Util = imports.misc.util;
const PanelMenu = imports.ui.panelMenu;
const Mainloop = imports.mainloop;
const GLib = imports.gi.GLib;
const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

//images
const wg_red = Gio.icon_new_for_string(Me.dir.get_path()+'/icons/wg-red.svg');
const wg_gray = Gio.icon_new_for_string(Me.dir.get_path()+'/icons/wg-gray.svg');
const wg_black = Gio.icon_new_for_string(Me.dir.get_path()+'/icons/wg-black.svg');

//timeout
const TIMEOUT_SECONDS = 2


const WgIndicator = new Lang.Class({
    Name: 'WgIndicator',
    Extends: PanelMenu.Button,

    _init: function() {
        this.parent(0.0, "WG Indicator", false);
        
        //icon
        this.wg_icon = new St.Icon({
            gicon: wg_black,
            style_class: "system-status-icon",
        });

        this.add_actor(this.wg_icon);
        this._refresh();
    },
    
    _checkWG: function() {
        //check for wireguard connections using nmcli

        //spawn process
        let process = Gio.Subprocess.new(
            ["/bin/bash", "-c", "nmcli c s --active | grep wireguard"],
            Gio.SubprocessFlags.NONE
        );

        //wait for process to finish
        let cancellable = new Gio.Cancellable();
        process.wait_async(cancellable, (process, result) => {
            //wait for process to finish
            process.wait_finish(result);
            
            //get status
            let status = process.get_status();
            
            //update indicator
            if (status === 256) { //disabled
                this.wg_icon.gicon = wg_gray;
            } else if (status === 0) { //enabled
                this.wg_icon.gicon = wg_red;
            } else { //error
                this.wg_icon.gicon = wg_black;
                getError(stdout);
            }
        });
    },

    _refresh: function() {
        this._checkWG()

        if (this._timeout) {
            Mainloop.source_remove(this._timeout);
            this._timeout = null;
        }

        this._timeout = Mainloop.timeout_add_seconds(TIMEOUT_SECONDS, Lang.bind(this, this._refresh));
    }
});

let twMenu;

function init() {
}

function enable() {
    twMenu = new WgIndicator;
    let wg_green = new St.Icon({
            gicon: Gio.icon_new_for_string(`icons/wireguard-green.svg`),
            style_class: "system-status-icon",
        });
    twMenu.icon = wg_green;
    Main.panel.addToStatusArea('wg-indicator', twMenu);
}

function disable() {
    if (twMenu && twMenu._timeout) {
        Mainloop.source_remove(twMenu._timeout);
    }

    twMenu.destroy();
    twMenu = null;
}
